@@ -5,12 +5,15 @@ from typing import Union
 
 import webbrowser
 
+# fmt: off
 from rasgoql.data import (
     DWCredentials,
     BigQueryCredentials, BigQueryDataWarehouse,
     SnowflakeCredentials, SnowflakeDataWarehouse,
-    PostgresCredentials, PostgresDataWarehouse
-)
+    PostgresCredentials, PostgresDataWarehouse,
+    MySQLCredentials, MySQLDataWarehouse,
+    RedshiftCredentials, RedshiftDataWarehouse
+)  # fmt: on
 from rasgoql.errors import ParameterException
 from rasgoql.main import RasgoQL
 from rasgoql.version import __version__
@@ -25,6 +28,8 @@ DW_MAP = {
     'snowflake': SnowflakeDataWarehouse,
     'bigquery': BigQueryDataWarehouse,
     'postgresql': PostgresDataWarehouse,
+    "mysql": MySQLDataWarehouse,
+    'redshift': RedshiftDataWarehouse,
 }
 
 
@@ -36,13 +41,14 @@ def docs():
     webbrowser.open(url)
     print(url)
 
+
 def connect(
-        credentials: DWCredentials
-    ):
+        credentials: DWCredentials,
+):
     """
     Return a RasgoQL object connected to a Data Warehouse
     """
     return RasgoQL(
         connection=DW_MAP[credentials.dw_type],
-        credentials=credentials.to_dict()
+        credentials=credentials.to_dict(),
     )
