@@ -2,3 +2,5 @@ from rasgoql.data.base import DataWarehouse, DWCredentials
 from rasgoql.data.bigquery import BigQueryDataWarehouse, BigQueryCredentials
 from rasgoql.data.snowflake import SnowflakeDataWarehouse, SnowflakeCredentials
 from rasgoql.data.postgres import PostgresCredentials, PostgresDataWarehouse
+from rasgoql.data.mysql import MySQLCredentials, MySQLDataWarehouse
+from rasgoql.data.redshift import RedshiftCredentials, RedshiftDataWarehouse
