@@ -4,30 +4,27 @@ BigQuery Data Warehouse classes
 
 import logging
 import os
-from typing import List, Tuple, Union
+from typing import List, Optional, Tuple, Union
 
 import json
 import pandas as pd
 
 from rasgoql.data.base import DataWarehouse, DWCredentials
 from rasgoql.errors import (
-    DWCredentialsWarning, DWConnectionError, DWQueryError,
-    ParameterException, PackageDependencyWarning,
-    SQLWarning, TableAccessError, TableConflictException
+    DWCredentialsWarning,
+    DWConnectionError,
+    DWQueryError,
+    PackageDependencyWarning,
+    SQLWarning,
+    TableAccessError,
+    TableConflictException,
 )
 from rasgoql.imports import bq, gcp_exc, gcp_flow, gcp_svc
-from rasgoql.primitives.enums import (
-    check_response_type,
-    check_write_method, check_write_table_type
-)
+from rasgoql.primitives.enums import check_response_type, check_write_method, check_write_table_type
 from rasgoql.utils.creds import load_env, save_env
 from rasgoql.utils.df import cleanse_sql_dataframe
 from rasgoql.utils.messaging import verbose_message
-from rasgoql.utils.sql import (
-    is_scary_sql, magic_fqtn_handler,
-    parse_fqtn, parse_namespace,
-    validate_namespace
-)
+from rasgoql.utils.sql import is_scary_sql
 
 logging.basicConfig()
 logger = logging.getLogger('BigQuery DataWarehouse')
@@ -38,14 +35,15 @@ class BigQueryCredentials(DWCredentials):
     """
     BigQuery Credentials
     """
+
     dw_type = 'bigquery'
 
     def __init__(
-            self,
-            json_filepath: str,
-            project: str = None,
-            dataset: str = None
-        ):
+        self,
+        json_filepath: str,
+        project: str = None,
+        dataset: str = None,
+    ):
         self.json_filepath = json_filepath
         self.project = project
         self.dataset = dataset
@@ -55,15 +53,15 @@ class BigQueryCredentials(DWCredentials):
             {
                 "json_filepath": self.json_filepath,
                 "project": self.project,
-                "dataset": self.dataset
+                "dataset": self.dataset,
             }
         )
 
     @classmethod
     def from_env(
-            cls,
-            filepath: str = None
-        ) -> 'BigQueryCredentials':
+        cls,
+        filepath: str = None,
+    ) -> 'BigQueryCredentials':
         """
         Creates an instance of this Class from a .env file on your machine
         """
@@ -79,7 +77,7 @@ class BigQueryCredentials(DWCredentials):
         return cls(
             json_filepath,
             project,
-            dataset
+            dataset,
         )
 
     def to_dict(self) -> dict:
@@ -89,14 +87,14 @@ class BigQueryCredentials(DWCredentials):
         return {
             "json_filepath": self.json_filepath,
             "project": self.project,
-            "dataset": self.dataset
+            "dataset": self.dataset,
         }
 
     def to_env(
-            self,
-            filepath: str = None,
-            overwrite: bool = False
-        ):
+        self,
+        filepath: str = None,
+        overwrite: bool = False,
+    ):
         """
         Saves credentials to a .env file on your machine
         """
@@ -112,6 +110,7 @@ class BigQueryDataWarehouse(DataWarehouse):
     """
     Google BigQuery DataWarehouse
     """
+
     dw_type = 'bigquery'
     credentials_class = BigQueryCredentials
 
@@ -120,7 +119,8 @@ class BigQueryDataWarehouse(DataWarehouse):
             raise PackageDependencyWarning(
                 'Missing a required python package to run BigQuery. '
                 'Please download the BigQuery package by running: '
-                'pip install rasgoql[bigquery]')
+                'pip install rasgoql[bigquery]'
+            )
 
         super().__init__()
         self.credentials: dict = None
@@ -132,9 +132,9 @@ class BigQueryDataWarehouse(DataWarehouse):
     # Core Data Warehouse methods
     # ---------------------------
     def change_namespace(
-            self,
-            namespace: str
-        ) -> None:
+        self,
+        namespace: str,
+    ) -> None:
         """
         Changes the default namespace of your connection
 
@@ -142,8 +142,8 @@ class BigQueryDataWarehouse(DataWarehouse):
         `namespace`: str:
             namespace (project.dataset)
         """
-        namespace = self._validate_namespace(namespace)
-        project, dataset = parse_namespace(namespace)
+        namespace = self.validate_namespace(namespace)
+        project, dataset = self.parse_namespace(namespace)
         try:
             self.execute_query(f'USE PROJECT {project}')
             self.execute_query(f'USE DATASET {dataset}')
@@ -152,15 +152,15 @@ class BigQueryDataWarehouse(DataWarehouse):
             self.default_dataset = dataset
             verbose_message(
                 f"Namespace reset to {self.default_namespace}",
-                logger
+                logger,
             )
         except Exception as e:
             self._error_handler(e)
 
     def connect(
-            self,
-            credentials: Union[dict, BigQueryCredentials]
-        ):
+        self,
+        credentials: Union[dict, BigQueryCredentials],
+    ):
         """
         Connect to BigQuery
 
@@ -175,16 +175,14 @@ class BigQueryDataWarehouse(DataWarehouse):
             self.default_project = credentials.get('project')
             self.default_dataset = credentials.get('dataset')
             if not self.credentials:
-                self.credentials = self._get_credentials(
-                    credentials.get('json_filepath')
-                )
+                self.credentials = self._get_credentials(credentials.get('json_filepath'))
             self.connection = bq.Client(
                 credentials=self.credentials,
-                project=self.default_project
+                project=self.default_project,
             )
             verbose_message(
                 "Connected to BigQuery",
-                logger
+                logger,
             )
         except Exception as e:
             self._error_handler(e)
@@ -199,18 +197,18 @@ class BigQueryDataWarehouse(DataWarehouse):
             self.connection = None
             verbose_message(
                 "Connection to BigQuery closed",
-                logger
+                logger,
             )
         except Exception as e:
             self._error_handler(e)
 
     def create(
-            self,
-            sql: str,
-            fqtn: str,
-            table_type: str = 'VIEW',
-            overwrite: bool = False
-        ):
+        self,
+        sql: str,
+        fqtn: str,
+        table_type: str = 'VIEW',
+        overwrite: bool = False,
+    ):
         """
         Create a view or table from given SQL
 
@@ -228,11 +226,13 @@ class BigQueryDataWarehouse(DataWarehouse):
             WARNING: This will completely overwrite data in the existing table
         """
         table_type = check_write_table_type(table_type)
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
         if self._table_exists(fqtn) and not overwrite:
-            msg = f'A table or view named {fqtn} already exists. ' \
-                   'If you are sure you want to overwrite it, ' \
-                   'pass in overwrite=True and run this function again'
+            msg = (
+                f'A table or view named {fqtn} already exists. '
+                'If you are sure you want to overwrite it, '
+                'pass in overwrite=True and run this function again'
+            )
             raise TableConflictException(msg)
         query = f'CREATE OR REPLACE {table_type} {fqtn} AS {sql}'
         self.execute_query(query, acknowledge_risk=True, response='None')
@@ -245,12 +245,26 @@ class BigQueryDataWarehouse(DataWarehouse):
         """
         return f'{self.default_project}.{self.default_dataset}'
 
+    @default_namespace.setter
+    def default_namespace(
+        self,
+        new_namespace: str,
+    ):
+        """
+        Setter method for the `default_namespace` property
+        """
+        namespace = self.validate_namespace(new_namespace)
+        project, dataset = self.parse_namespace(namespace)
+        self.default_project = project
+        self.default_dataset = dataset
+
     def execute_query(
-            self,
-            sql: str,
-            response: str = 'tuple',
-            acknowledge_risk: bool = False
-        ):
+        self,
+        sql: str,
+        response: str = 'tuple',
+        acknowledge_risk: bool = False,
+        **kwargs,
+    ):
         """
         Run a query against BigQuery and return all results
 
@@ -265,14 +279,16 @@ class BigQueryDataWarehouse(DataWarehouse):
         """
         response = check_response_type(response)
         if is_scary_sql(sql) and not acknowledge_risk:
-            msg = 'It looks like your SQL statement contains a ' \
-                  'potentially dangerous or data-altering operation.' \
-                  'If you are positive you want to run this, ' \
-                  'pass in acknowledge_risk=True and run this function again.'
+            msg = (
+                'It looks like your SQL statement contains a '
+                'potentially dangerous or data-altering operation.'
+                'If you are positive you want to run this, '
+                'pass in acknowledge_risk=True and run this function again.'
+            )
             raise SQLWarning(msg)
         verbose_message(
             f"Executing query: {sql}",
-            logger
+            logger,
         )
         if response == 'DICT':
             return self._query_into_dict(sql)
@@ -281,25 +297,25 @@ class BigQueryDataWarehouse(DataWarehouse):
         return self._execute_string(sql, ignore_results=(response == 'NONE'))
 
     def get_ddl(
-            self,
-            fqtn: str
-        ) -> str:
+        self,
+        fqtn: str,
+    ) -> str:
         """
         Returns the create statement for a table or view
 
         `fqtn`: str:
             Fully-qualified Table Name (database.schema.table)
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
-        proj, ds, tbl = parse_fqtn(fqtn)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
+        proj, ds, tbl = self.parse_fqtn(fqtn)
         sql = f"SELECT DDL FROM {proj}.{ds}.INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='{tbl}'"
         query_response = self.execute_query(sql)
         return query_response[0]
 
     def get_object_details(
-            self,
-            fqtn: str
-        ) -> tuple:
+        self,
+        fqtn: str,
+    ) -> tuple:
         """
         Return details of a table or view in BigQuery
 
@@ -312,7 +328,7 @@ class BigQueryDataWarehouse(DataWarehouse):
             is rasgo object: bool
             object type: [table|view|unknown]
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
         obj_exists = False
         is_rasgo_obj = False
         obj_type = 'unknown'
@@ -329,10 +345,10 @@ class BigQueryDataWarehouse(DataWarehouse):
             return obj_exists, is_rasgo_obj, obj_type
 
     def get_schema(
-            self,
-            fqtn: str,
-            create_sql: str = None
-        ) -> Tuple[str, str]:
+        self,
+        fqtn: str,
+        create_sql: str = None,
+    ) -> List[Tuple[str, str]]:
         """
         Return the schema of a table or view
 
@@ -344,7 +360,7 @@ class BigQueryDataWarehouse(DataWarehouse):
             and the fqtn does not already exist, it will be created and profiled based
             on this statement. The view will be dropped after profiling
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
         response = []
         try:
             if self._table_exists(fqtn):
@@ -362,10 +378,10 @@ class BigQueryDataWarehouse(DataWarehouse):
             self._error_handler(e)
 
     def list_tables(
-            self,
-            database: str = None,
-            schema: str = None
-        ) -> pd.DataFrame:
+        self,
+        database: str = None,
+        schema: str = None,
+    ) -> pd.DataFrame:
         """
         List all tables and views available in default namespace
 
@@ -387,7 +403,7 @@ class BigQueryDataWarehouse(DataWarehouse):
                 'TABLE_TYPE',
                 'ROW_COUNT',
                 'CREATED',
-                'LAST_ALTERED'
+                'LAST_ALTERED',
             ]
             for tbl in tables:
                 table = self.connection.get_table(tbl)
@@ -398,21 +414,21 @@ class BigQueryDataWarehouse(DataWarehouse):
                         table.table_type,
                         table.num_rows,
                         table.created,
-                        table.modified
+                        table.modified,
                     )
                 )
             return pd.DataFrame(
                 records,
-                columns=columns
+                columns=columns,
             )
         except Exception as e:
             self._error_handler(e)
 
     def preview(
-            self,
-            sql: str,
-            limit: int = 10
-        ) -> pd.DataFrame:
+        self,
+        sql: str,
+        limit: int = 10,
+    ) -> pd.DataFrame:
         """
         Returns 10 records into a pandas DataFrame
 
@@ -425,15 +441,15 @@ class BigQueryDataWarehouse(DataWarehouse):
         return self.execute_query(
             f'{sql} LIMIT {limit}',
             response='df',
-            acknowledge_risk=True
+            acknowledge_risk=True,
         )
 
     def save_df(
-            self,
-            df: pd.DataFrame,
-            fqtn: str,
-            method: str = None
-        ) -> str:
+        self,
+        df: pd.DataFrame,
+        fqtn: str,
+        method: str = None,
+    ) -> str:
         """
         Creates a table in BigQuery from a pandas Dataframe
 
@@ -450,22 +466,22 @@ class BigQueryDataWarehouse(DataWarehouse):
         """
         if method:
             method = check_write_method(method)
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
         table_exists = self._table_exists(fqtn)
         if table_exists and not method:
-            msg = f"A table named {fqtn} already exists. " \
-                   "If you are sure you want to write over it, pass in " \
-                   "method='append' or method='replace' and run this function again"
+            msg = (
+                f"A table named {fqtn} already exists. "
+                "If you are sure you want to write over it, pass in "
+                "method='append' or method='replace' and run this function again"
+            )
             raise TableConflictException(msg)
         try:
             cleanse_sql_dataframe(df)
-            job_config = bq.LoadJobConfig(
-                write_disposition='WRITE_TRUNCATE' if method == 'REPLACE' else None
-            )
+            job_config = bq.LoadJobConfig(write_disposition='WRITE_TRUNCATE' if method == 'REPLACE' else None)
             _job = self.connection.load_table_from_dataframe(
                 df,
                 fqtn,
-                job_config=job_config
+                job_config=job_config,
             )
             # Wait for the job to complete
             _job.result()
@@ -473,14 +489,13 @@ class BigQueryDataWarehouse(DataWarehouse):
         except Exception as e:
             self._error_handler(e)
 
-
     # ---------------------------
     # Core Data Warehouse helpers
     # ---------------------------
     def _table_exists(
-            self,
-            fqtn: str
-        ) -> bool:
+        self,
+        fqtn: str,
+    ) -> bool:
         """
         Check for existence of fqtn in the Data Warehouse and return a boolean
 
@@ -488,52 +503,29 @@ class BigQueryDataWarehouse(DataWarehouse):
         `fqtn`: str:
             Fully-qualified table name (database.schema.table)
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
         try:
             self.connection.get_table(fqtn)
             return True
         except gcp_exc.NotFound:
             return False
 
-    def _validate_namespace(
-            self,
-            namespace: str
-        ) -> str:
-        """
-        Checks a namespace string for compliance with BigQuery format
-
-        Params:
-        `namespace`: str:
-            namespace (project.dataset)
-        """
-        # Does this match a 'string.string' pattern?
-        try:
-            validate_namespace(namespace)
-            return namespace
-        except ValueError:
-            raise ParameterException("Bigquery namespaces should be format: PROJECT.DATASET")
-
     # --------------------------
     # BigQuery specific helpers
     # --------------------------
     @property
     def _default_job_config(self) -> 'bq.QueryJobConfig':
-        return bq.QueryJobConfig(
-            default_dataset=self.default_namespace
-            )
+        return bq.QueryJobConfig(default_dataset=self.default_namespace)
 
     def _error_handler(
-            self,
-            exception: Exception,
-            query: str = None
-        ) -> None:
+        self,
+        exception: Exception,
+        query: str = None,
+    ) -> None:
         """
         Handle Snowflake exceptions that need additional info
         """
-        verbose_message(
-            f'Exception occurred while running query: {query}',
-            logger
-        )
+        verbose_message(f'Exception occurred while running query: {query}', logger),
         if exception is None:
             return
         if isinstance(exception, gcp_exc.NotFound):
@@ -552,17 +544,17 @@ class BigQueryDataWarehouse(DataWarehouse):
         raise exception
 
     def _execute_string(
-            self,
-            query: str,
-            ignore_results: bool = False
-        ) -> List[tuple]:
+        self,
+        query: str,
+        ignore_results: bool = False,
+    ) -> Optional[List[tuple]]:
         """
         Execute a query string against the DataWarehouse connection and fetch all results
         """
         try:
             query_job = self.connection.query(
                 query,
-                job_config=self._default_job_config
+                job_config=self._default_job_config,
             )
             if ignore_results:
                 return
@@ -571,73 +563,77 @@ class BigQueryDataWarehouse(DataWarehouse):
             self._error_handler(e)
 
     def _get_appflow_credentials(
-            self,
-            filepath: str
-        ) -> 'bq.Credentials':
+        self,
+        filepath: str,
+    ) -> 'bq.Credentials':
         try:
             appflow = gcp_flow.InstalledAppFlow.from_client_secrets_file(
                 filepath,
-                scopes=["https://www.googleapis.com/auth/bigquery"]
+                scopes=["https://www.googleapis.com/auth/bigquery"],
             )
             appflow.run_local_server()
-            #appflow.run_console()
+            # appflow.run_console()
             return appflow.credentials
         except Exception as e:
             self._error_handler(e)
 
     def _get_credentials(
-            self,
-            filepath: str
-        ) -> 'bq.Credentials':
+        self,
+        filepath: str,
+    ) -> 'bq.Credentials':
         try:
             return self._get_appflow_credentials(filepath)
         except ValueError:
             return self._get_service_credentials(filepath)
 
     def _get_service_credentials(
-            self,
-            filepath: str
-        ) -> 'bq.Credentials':
+        self,
+        filepath: str,
+    ) -> 'bq.Credentials':
         try:
             credentials = gcp_svc.Credentials.from_service_account_file(
                 filepath,
-                scopes=["https://www.googleapis.com/auth/cloud-platform"]
+                scopes=["https://www.googleapis.com/auth/cloud-platform"],
             )
             return credentials
         except Exception as e:
             self._error_handler(e)
 
     def _query_into_dict(
-            self,
-            query: str
+        self,
+        query: str,
     ) -> dict:
         """
         Return results of query in a pandas DataFrame
         """
         try:
-            return self.connection.query(
-                query,
-                job_config=self._default_job_config
-                ) \
-                .result() \
-                .to_dataframe() \
+            return (
+                self.connection.query(
+                    query,
+                    job_config=self._default_job_config,
+                )
+                .result()
+                .to_dataframe()
                 .to_dict()
+            )
         except Exception as e:
             self._error_handler(e)
 
     def _query_into_pandas(
-            self,
-            query: str
+        self,
+        query: str,
     ) -> pd.DataFrame:
         """
         Return results of query in a pandas DataFrame
         """
         try:
-            return self.connection.query(
-                query,
-                job_config=self._default_job_config
-                ) \
-                .result() \
+            return (
+                self.connection.query(
+                    query,
+                    job_config=self._default_job_config,
+                )
+                .result()
                 .to_dataframe()
+            )
         except Exception as e:
             self._error_handler(e)
