@@ -1,31 +1,31 @@
 """
 Snowflake DataWarehouse classes
 """
+from __future__ import annotations
 import logging
 import os
-from typing import List, Optional, Union
+from typing import List, Optional, Tuple, Union
 
 import json
 import pandas as pd
 
 from rasgoql.data.base import DataWarehouse, DWCredentials
 from rasgoql.errors import (
-    DWCredentialsWarning, DWConnectionError, DWQueryError,
-    PackageDependencyWarning, ParameterException,
-    SQLWarning, TableAccessError, TableConflictException
+    DWCredentialsWarning,
+    DWConnectionError,
+    DWQueryError,
+    PackageDependencyWarning,
+    ParameterException,
+    SQLWarning,
+    TableAccessError,
+    TableConflictException,
 )
 from rasgoql.imports import sf_connector, write_pandas
-from rasgoql.primitives.enums import (
-    check_response_type, check_write_method, check_write_table_type
-)
+from rasgoql.primitives.enums import check_response_type, check_write_method, check_write_table_type
 from rasgoql.utils.creds import load_env, save_env
 from rasgoql.utils.df import cleanse_sql_dataframe, generate_dataframe_ddl
 from rasgoql.utils.messaging import verbose_message
-from rasgoql.utils.sql import (
-    is_scary_sql, magic_fqtn_handler,
-    parse_fqtn, parse_namespace,
-    validate_namespace
-)
+from rasgoql.utils.sql import is_scary_sql
 
 logging.basicConfig()
 logger = logging.getLogger('Snowflake DataWarehouse')
@@ -36,23 +36,25 @@ class SnowflakeCredentials(DWCredentials):
     """
     Snowflake Credentials
     """
+
     dw_type = 'snowflake'
 
     def __init__(
-            self,
-            account: str,
-            user: str,
-            password: str,
-            role: str,
-            warehouse: str,
-            database: str,
-            schema: str
-        ):
+        self,
+        account: str,
+        user: str,
+        password: str,
+        role: str,
+        warehouse: str,
+        database: str,
+        schema: str,
+    ):
         if sf_connector is None:
             raise PackageDependencyWarning(
                 'Missing a required python package to run Snowflake. '
                 'Please download the Snowflake package by running: '
-                'pip install rasgoql[snowflake]')
+                'pip install rasgoql[snowflake]'
+            )
         self.account = account
         self.user = user
         self.password = password
@@ -75,9 +77,9 @@ class SnowflakeCredentials(DWCredentials):
 
     @classmethod
     def from_env(
-            cls,
-            filepath: str = None
-        ) -> 'SnowflakeCredentials':
+        cls,
+        filepath: str = None,
+    ) -> SnowflakeCredentials:
         """
         Creates an instance of this Class from a .env file on your machine
         """
@@ -101,7 +103,7 @@ class SnowflakeCredentials(DWCredentials):
             role,
             warehouse,
             database,
-            schema
+            schema,
         )
 
     def to_dict(self) -> dict:
@@ -115,14 +117,14 @@ class SnowflakeCredentials(DWCredentials):
             "role": self.role,
             "warehouse": self.warehouse,
             "database": self.database,
-            "schema": self.schema
+            "schema": self.schema,
         }
 
     def to_env(
-            self,
-            filepath: str = None,
-            overwrite: bool = False
-        ):
+        self,
+        filepath: str = None,
+        overwrite: bool = False,
+    ):
         """
         Saves credentials to a .env file on your machine
         """
@@ -142,6 +144,7 @@ class SnowflakeDataWarehouse(DataWarehouse):
     """
     Snowflake DataWarehouse
     """
+
     dw_type = 'snowflake'
     credentials_class = SnowflakeCredentials
 
@@ -156,9 +159,9 @@ class SnowflakeDataWarehouse(DataWarehouse):
     # Core Data Warehouse methods
     # ---------------------------
     def change_namespace(
-            self,
-            namespace: str
-        ) -> None:
+        self,
+        namespace: str,
+    ) -> None:
         """
         Changes the default namespace of your connection
 
@@ -166,8 +169,8 @@ class SnowflakeDataWarehouse(DataWarehouse):
         `namespace`: str:
             namespace (database.schema)
         """
-        namespace = self._validate_namespace(namespace)
-        database, schema = parse_namespace(namespace)
+        namespace = self.validate_namespace(namespace)
+        database, schema = self.parse_namespace(namespace)
         try:
             self.execute_query(f'USE DATABASE {database}')
             self.execute_query(f'USE SCHEMA {schema}')
@@ -176,15 +179,15 @@ class SnowflakeDataWarehouse(DataWarehouse):
             self.default_schema = schema
             verbose_message(
                 f"Namespace reset to {self.default_namespace}",
-                logger
+                logger,
             )
         except Exception as e:
             self._error_handler(e)
 
     def connect(
-            self,
-            credentials: Union[dict, SnowflakeCredentials]
-        ):
+        self,
+        credentials: Union[dict, SnowflakeCredentials],
+    ):
         """
         Connect to Snowflake
 
@@ -196,12 +199,12 @@ class SnowflakeDataWarehouse(DataWarehouse):
             credentials = credentials.to_dict()
 
         # This allows you to track what queries were run by RasgoQL in your history tab
-        credentials.update({
-            "application": "rasgoql",
-            "session_parameters": {
-                "QUERY_TAG": "rasgoql"
+        credentials.update(
+            {
+                "application": "rasgoql",
+                "session_parameters": {"QUERY_TAG": "rasgoql"},
             }
-        })
+        )
         try:
             self.credentials = credentials
             self.default_database = credentials.get('database')
@@ -209,7 +212,7 @@ class SnowflakeDataWarehouse(DataWarehouse):
             self.connection = sf_connector.connect(**credentials)
             verbose_message(
                 "Connected to Snowflake",
-                logger
+                logger,
             )
         except Exception as e:
             self._error_handler(e)
@@ -224,18 +227,18 @@ class SnowflakeDataWarehouse(DataWarehouse):
             self.connection = None
             verbose_message(
                 "Connection to Snowflake closed",
-                logger
+                logger,
             )
         except Exception as e:
             self._error_handler(e)
 
     def create(
-            self,
-            sql: str,
-            fqtn: str,
-            table_type: str = 'VIEW',
-            overwrite: bool = False
-        ):
+        self,
+        sql: str,
+        fqtn: str,
+        table_type: str = 'VIEW',
+        overwrite: bool = False,
+    ):
         """
         Create a view or table from given SQL
 
@@ -253,11 +256,13 @@ class SnowflakeDataWarehouse(DataWarehouse):
             WARNING: This will completely overwrite data in the existing table
         """
         table_type = check_write_table_type(table_type)
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
         if self._table_exists(fqtn) and not overwrite:
-            msg = f'A table or view named {fqtn} already exists. ' \
-                   'If you are sure you want to overwrite it, ' \
-                   'pass in overwrite=True and run this function again'
+            msg = (
+                f'A table or view named {fqtn} already exists. '
+                'If you are sure you want to overwrite it, '
+                'pass in overwrite=True and run this function again'
+            )
             raise TableConflictException(msg)
         query = f"CREATE OR REPLACE {table_type} {fqtn} COMMENT='rasgoql' AS {sql}"
         self.execute_query(query, acknowledge_risk=True, response='None')
@@ -270,12 +275,26 @@ class SnowflakeDataWarehouse(DataWarehouse):
         """
         return f'{self.default_database}.{self.default_schema}'
 
+    @default_namespace.setter
+    def default_namespace(
+        self,
+        new_namespace: str,
+    ):
+        """
+        Setter method for the `default_namespace` property
+        """
+        namespace = self.validate_namespace(new_namespace)
+        db, schema = self.parse_namespace(namespace)
+        self.default_database = db
+        self.default_schema = schema
+
     def execute_query(
-            self,
-            sql: str,
-            response: str = 'tuple',
-            acknowledge_risk: bool = False
-        ):
+        self,
+        sql: str,
+        response: str = 'tuple',
+        acknowledge_risk: bool = False,
+        batches: bool = False,
+    ):
         """
         Run a query against Snowflake and return all results
 
@@ -290,40 +309,42 @@ class SnowflakeDataWarehouse(DataWarehouse):
         """
         response = check_response_type(response)
         if is_scary_sql(sql) and not acknowledge_risk:
-            msg = 'It looks like your SQL statement contains a ' \
-                  'potentially dangerous or data-altering operation.' \
-                  'If you are positive you want to run this, ' \
-                  'pass in acknowledge_risk=True and run this function again.'
+            msg = (
+                'It looks like your SQL statement contains a '
+                'potentially dangerous or data-altering operation.'
+                'If you are positive you want to run this, '
+                'pass in acknowledge_risk=True and run this function again.'
+            )
             raise SQLWarning(msg)
         verbose_message(
             f"Executing query: {sql}",
-            logger
+            logger,
         )
         if response == 'DICT':
             return self._execute_dict_cursor(sql)
         if response == 'DF':
-            return self._execute_df_cursor(sql)
+            return self._execute_df_cursor(sql, batches=batches)
         return self._execute_string(sql, ignore_results=(response == 'NONE'))
 
     def get_ddl(
-            self,
-            fqtn: str
-        ) -> str:
+        self,
+        fqtn: str,
+    ) -> str:
         """
         Returns the create statement for a table or view
 
         `fqtn`: str:
             Fully-qualified Table Name (database.schema.table)
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
         sql = f"SELECT GET_DDL('TABLE', '{fqtn}') AS DDL"
         query_response = self.execute_query(sql, response='dict')
         return query_response[0]['DDL']
 
     def get_object_details(
-            self,
-            fqtn: str
-        ) -> tuple:
+        self,
+        fqtn: str,
+    ) -> tuple:
         """
         Return details of a table or view in Snowflake
 
@@ -336,23 +357,23 @@ class SnowflakeDataWarehouse(DataWarehouse):
             is rasgo object: bool
             object type: [table|view|unknown]
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
-        database, schema, table = parse_fqtn(fqtn)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
+        database, schema, table = self.parse_fqtn(fqtn)
         sql = f"SHOW OBJECTS LIKE '{table}' IN {database}.{schema}"
         result = self.execute_query(sql, response='dict')
         obj_exists = len(result) > 0
         is_rasgo_obj = False
         obj_type = 'unknown'
         if obj_exists:
-            is_rasgo_obj = (result[0].get('comment') == 'rasgoql')
+            is_rasgo_obj = result[0].get('comment') == 'rasgoql'
             obj_type = result[0].get('kind')
         return obj_exists, is_rasgo_obj, obj_type
 
     def get_schema(
-            self,
-            fqtn: str,
-            create_sql: str = None
-        ) -> dict:
+        self,
+        fqtn: str,
+        create_sql: str = None,
+    ) -> List[Tuple[str, str]]:
         """
         Return the schema of a table or view
 
@@ -364,7 +385,7 @@ class SnowflakeDataWarehouse(DataWarehouse):
             and the fqtn does not already exist, it will be created and profiled based
             on this statement. The view will be dropped after profiling
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
         desc_sql = f"DESC TABLE {fqtn}"
         response = []
         try:
@@ -383,10 +404,10 @@ class SnowflakeDataWarehouse(DataWarehouse):
             self._error_handler(e)
 
     def list_tables(
-            self,
-            database: str = None,
-            schema: str = None
-        ) -> pd.DataFrame:
+        self,
+        database: str = None,
+        schema: str = None,
+    ) -> pd.DataFrame:
         """
         List all tables and views available in default namespace
 
@@ -396,10 +417,12 @@ class SnowflakeDataWarehouse(DataWarehouse):
         `schema`: str:
             override schema
         """
-        select_clause = "SELECT TABLE_NAME, " \
-                        "TABLE_CATALOG||'.'||TABLE_SCHEMA||'.'||TABLE_NAME AS FQTN, " \
-                        "CASE TABLE_TYPE WHEN 'BASE TABLE' THEN 'TABLE' ELSE TABLE_TYPE END AS TABLE_TYPE, " \
-                        "ROW_COUNT, CREATED, LAST_ALTERED "
+        select_clause = (
+            "SELECT TABLE_NAME, "
+            "TABLE_CATALOG||'.'||TABLE_SCHEMA||'.'||TABLE_NAME AS FQTN, "
+            "CASE TABLE_TYPE WHEN 'BASE TABLE' THEN 'TABLE' ELSE TABLE_TYPE END AS TABLE_TYPE, "
+            "ROW_COUNT, CREATED, LAST_ALTERED "
+        )
         from_clause = " FROM INFORMATION_SCHEMA.TABLES "
         if database:
             from_clause = f" FROM {database.upper()}.INFORMATION_SCHEMA.TABLES "
@@ -408,10 +431,10 @@ class SnowflakeDataWarehouse(DataWarehouse):
         return self.execute_query(sql, response='df', acknowledge_risk=True)
 
     def preview(
-            self,
-            sql: str,
-            limit: int = 10
-        ) -> pd.DataFrame:
+        self,
+        sql: str,
+        limit: int = 10,
+    ) -> pd.DataFrame:
         """
         Returns 10 records into a pandas DataFrame
 
@@ -424,15 +447,15 @@ class SnowflakeDataWarehouse(DataWarehouse):
         return self.execute_query(
             f'{sql} LIMIT {limit}',
             response='df',
-            acknowledge_risk=True
+            acknowledge_risk=True,
         )
 
     def save_df(
-            self,
-            df: pd.DataFrame,
-            fqtn: str,
-            method: str = None
-        ) -> str:
+        self,
+        df: pd.DataFrame,
+        fqtn: str,
+        method: str = None,
+    ) -> str:
         """
         Creates a table in Snowflake from a pandas Dataframe
 
@@ -451,12 +474,14 @@ class SnowflakeDataWarehouse(DataWarehouse):
         """
         if method:
             method = check_write_method(method)
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
         table_exists = self._table_exists(fqtn)
         if table_exists and not method:
-            msg = f"A table named {fqtn} already exists. " \
-                   "If you are sure you want to write over it, pass in " \
-                   "method='append' or method='replace' and run this function again"
+            msg = (
+                f"A table named {fqtn} already exists. "
+                "If you are sure you want to write over it, pass in "
+                "method='append' or method='replace' and run this function again"
+            )
             raise TableConflictException(msg)
         try:
             cleanse_sql_dataframe(df)
@@ -470,7 +495,7 @@ class SnowflakeDataWarehouse(DataWarehouse):
                 conn=self.connection,
                 df=df,
                 table_name=fqtn,
-                quote_identifiers=False
+                quote_identifiers=False,
             )
             return fqtn
         except Exception as e:
@@ -480,9 +505,9 @@ class SnowflakeDataWarehouse(DataWarehouse):
     # Core Data Warehouse helpers
     # ---------------------------
     def _table_exists(
-            self,
-            fqtn: str
-        ) -> bool:
+        self,
+        fqtn: str,
+    ) -> bool:
         """
         Check for existence of fqtn in the Data Warehouse and return a boolean
 
@@ -490,41 +515,24 @@ class SnowflakeDataWarehouse(DataWarehouse):
         `fqtn`: str:
             Fully-qualified table name (database.schema.table)
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        fqtn = self.magic_fqtn_handler(fqtn, self.default_namespace)
         do_i_exist, _, _ = self.get_object_details(fqtn)
         return do_i_exist
 
-    def _validate_namespace(
-            self,
-            namespace: str
-        ) -> str:
-        """
-        Checks a namespace string for compliance with Snowflake format
-
-        Params:
-        `namespace`: str:
-            namespace (database.schema)
-        """
-        try:
-            validate_namespace(namespace)
-            return namespace.upper()
-        except ValueError:
-            raise ParameterException("Snowflake namespaces should be format: DATABASE.SCHEMA")
-
     # --------------------------
     # Snowflake specific helpers
     # --------------------------
     def _error_handler(
-            self,
-            exception: Exception,
-            query: str = None
-        ) -> None:
+        self,
+        exception: Exception,
+        query: str = None,
+    ) -> None:
         """
         Handle Snowflake exceptions that need additional info
         """
         verbose_message(
             f"Exception occurred while running query: {query}",
-            logger
+            logger,
         )
         if exception is None:
             return
@@ -553,9 +561,9 @@ class SnowflakeDataWarehouse(DataWarehouse):
         raise exception
 
     def _execute_dict_cursor(
-            self,
-            query: str
-        ) -> List[dict]:
+        self,
+        query: str,
+    ) -> List[dict]:
         """
         Run a query string and return results in a Snowflake DictCursor
 
@@ -579,18 +587,21 @@ class SnowflakeDataWarehouse(DataWarehouse):
                 cursor.close()
 
     def _execute_df_cursor(
-            self,
-            query: str,
-            params: Optional[dict] = None
-        ) -> pd.DataFrame:
+        self,
+        query: str,
+        params: Optional[dict] = None,
+        batches: Optional[bool] = False,
+    ) -> pd.DataFrame:
         """
         Run a query string and return results in a pandas DataFrame
         """
         cursor = None
         try:
             cursor = self.connection.cursor()
-            query_return = cursor.execute(query, params).fetch_pandas_all()
-            return query_return
+            cursor.execute(query, params)
+            if batches:
+                return cursor.fetch_pandas_batches()
+            return cursor.fetch_pandas_all()
         except Exception as e:
             self._error_handler(e)
         finally:
@@ -598,10 +609,10 @@ class SnowflakeDataWarehouse(DataWarehouse):
                 cursor.close()
 
     def _execute_string(
-            self,
-            query: str,
-            ignore_results: bool = False
-        ) -> List[tuple]:
+        self,
+        query: str,
+        ignore_results: bool = False,
+    ) -> List[tuple]:
         """
         Execute a query string against the Data Warehouse connection and fetch all results
         """
