@@ -6,7 +6,7 @@ import inspect
 from itertools import combinations, permutations, product
 import re
 import os
-from typing import Callable, Dict, List, Tuple, Optional
+from typing import Callable, Dict, List, Tuple, Optional, TYPE_CHECKING
 
 import jinja2
 import pandas as pd
@@ -16,14 +16,18 @@ from rasgoql.errors import TransformRenderingError
 from rasgoql.primitives.enums import check_write_table_type
 from rasgoql.utils.dbt import save_model_file
 
+if TYPE_CHECKING:
+    from rasgoql.primitives.transforms import Transform, TransformTemplate
+    from rasgoql.data.base import DataWarehouse
+
 RUN_QUERY_LIMIT = 100
 JINJA_ENV = jinja2.Environment(extensions=['jinja2.ext.do', 'jinja2.ext.loopcontrols'])
 
 
 def assemble_cte_chain(
-        transforms: List['Transform'],
-        table_type: str = None
-    ):
+    transforms: List['Transform'],
+    table_type: str = None,
+):
     """
     Returns a nested CTE statement to render input Transforms
     """
@@ -38,7 +42,7 @@ def assemble_cte_chain(
             t.arguments,
             t.source_table,
             None,
-            t._dw
+            t._dw,
         )
         return create_stmt + final_select
 
@@ -51,7 +55,7 @@ def assemble_cte_chain(
             t.arguments,
             t.source_table,
             running_sql,
-            t._dw
+            t._dw,
         )
 
         # Set final select & create statements from terminal transform
@@ -64,9 +68,10 @@ def assemble_cte_chain(
             t_list.append(t_cte_str)
     return create_stmt + 'WITH ' + ', \n'.join(t_list) + final_select
 
+
 def assemble_view_chain(
-        transforms: List['Transform']
-    ):
+    transforms: List['Transform'],
+):
     """
     Returns a nested view statement to render input Transforms
     """
@@ -79,7 +84,7 @@ def assemble_view_chain(
             t.arguments,
             t.source_table,
             running_sql,
-            t._dw
+            t._dw,
         )
         running_sql = _construct_running_sql(cte_list, t_sql)
         t_cte_str = f'{t.output_alias} AS (\n{t_sql}\n) '
@@ -88,13 +93,14 @@ def assemble_view_chain(
         view_list.append(t_view_str)
     return '\n'.join(view_list)
 
+
 def create_dbt_files(
-        transforms: List['Transform'],
-        schema: List[Tuple[str, str]],
-        output_directory: str = None,
-        file_name: str = None,
-        config_args: dict = None,
-        include_schema: bool = False
+    transforms: List['Transform'],
+    schema: List[Tuple[str, str]],
+    output_directory: str = None,
+    file_name: str = None,
+    config_args: dict = None,
+    include_schema: bool = False,
 ) -> str:
     """
     Saves a dbt_project.yml and model.sql files to a directory
@@ -107,16 +113,17 @@ def create_dbt_files(
         file_name=file_name,
         config_args=config_args,
         include_schema=include_schema,
-        schema=schema
+        schema=schema,
     )
 
+
 def generate_transform_sql(
-        name: str,
-        arguments: dict,
-        source_table: str = None,
-        running_sql: str = None,
-        dw: 'DataWarehouse' = None
-    ) -> str:
+    name: str,
+    arguments: dict,
+    source_table: str = None,
+    running_sql: str = None,
+    dw: 'DataWarehouse' = None,
+) -> str:
     """
     Returns the SQL for a Transform with applied arguments
     """
@@ -129,20 +136,22 @@ def generate_transform_sql(
         if udt.name == 'apply':
             source_code = arguments.pop('sql')
             if not source_code:
-                raise TransformRenderingError(f'Custom transform {udt.name} must provide ' \
-                                               'the "sql" argument with template to apply')
+                raise TransformRenderingError(
+                    f'Custom transform {udt.name} must provide the "sql" argument with template to apply'
+                )
         return render_template(name, source_code, arguments, source_table, running_sql, dw)
     except Exception as e:
         raise TransformRenderingError(e) from e
 
+
 def render_template(
-        name: str,
-        source_code: str,
-        arguments: dict,
-        source_table: str,
-        running_sql: str,
-        dw: 'DataWarehouse'
-    ) -> str:
+    name: str,
+    source_code: str,
+    arguments: dict,
+    source_table: str,
+    running_sql: str,
+    dw: 'DataWarehouse',
+) -> str:
     """
     Returns a SQL statement generated by applying arguments to a Jinja template
     """
@@ -150,13 +159,15 @@ def render_template(
     template = JINJA_ENV.from_string(source_code)
     rendered = template.render(**arguments, **template_fns)
     if not rendered:
-        raise TransformRenderingError(f'Rendering of transform {name} produced an empty string. ' \
-                                       'Please check template arguments and try again.')
+        raise TransformRenderingError(
+            f'Rendering of transform {name} produced an empty string. Please check template arguments and try again.'
+        )
     return rendered
 
+
 def _cleanse_template_symbol(
-        symbol: str
-    ) -> str:
+    symbol: str,
+) -> str:
     """
     Extra verbose function for clarity
 
@@ -171,12 +182,13 @@ def _cleanse_template_symbol(
     symbol = symbol.replace(' ', '_').replace('-', '_')
     symbol = symbol.upper()
     symbol = re.sub('[^A-Z0-9_]+', '', symbol)
-    symbol = '_'+symbol if symbol[0].isdecimal() or not symbol else symbol
+    symbol = '_' + symbol if symbol[0].isdecimal() or not symbol else symbol
     return symbol
 
+
 def _collapse_cte(
-        sql: str
-    ) -> str:
+    sql: str,
+) -> str:
     """
     Returns a collapsed CTE if sql is a CTE, or original sql
     """
@@ -184,10 +196,11 @@ def _collapse_cte(
         return re.sub(r'^(WITH)\s', r', ', sql, 1, flags=re.IGNORECASE)
     return sql
 
+
 def _construct_running_sql(
-        cte_list: List[str],
-        sql: str
-    ) -> str:
+    cte_list: List[str],
+    sql: str,
+) -> str:
     """
     Constructs and returns a running sql statement
     """
@@ -198,9 +211,10 @@ def _construct_running_sql(
         return sql
     return 'WITH ' + ', \n'.join(cte_list) + _collapse_cte(sql)
 
+
 def _gen_udt_func_docstring(
-        transform: 'TransformTemplate'
-    ) -> str:
+    transform: 'TransformTemplate',
+) -> str:
     """
     Generate and return a docstring for a transform func
     with transform description, args, and return specified.
@@ -212,14 +226,16 @@ def _gen_udt_func_docstring(
         docstring = f"{docstring}\n    {t_arg.get('name')}: {t_arg.get('description')}"
     docstring = f"{docstring}\n    operation_name: Name to set for the operation"
 
-    docstring = f"{docstring}\n\n  Returns:\n    Returns an new dataset with the referenced " \
-                f"{transform.name!r} added to this dataset's definition"
-    return docstring
+    return (
+        f"{docstring}\n\n  Returns:\n    Returns an new dataset with the referenced "
+        f"{transform.name!r} added to this dataset's definition"
+    )
+
 
 def _gen_udt_func_signature(
-        udt_func: Callable,
-        transform: 'TransformTemplate'
-    ) -> inspect.Signature:
+    udt_func: Callable,
+    transform: 'TransformTemplate',
+) -> inspect.Signature:
     """
     Creates and returns a UDT param signature.
 
@@ -236,25 +252,27 @@ def _gen_udt_func_signature(
         name='operation_name',
         kind=inspect.Parameter.KEYWORD_ONLY,
         annotation=Optional[str],
-        default=None
+        default=None,
     )
     udt_params.append(op_name_param)
     return sig.replace(parameters=udt_params)
 
+
 def _raise_exception(
-        message: str
-    ) -> None:
+    message: str,
+) -> None:
     """
     Raise an exception to return to users of a template
     """
     raise TransformRenderingError(message)
 
+
 def _run_query(
-        query: str,
-        source_table: str = None,
-        running_sql: str = None,
-        dw: 'DataWarehouse' = None
-    ) -> pd.DataFrame:
+    query: str,
+    source_table: str = None,
+    running_sql: str = None,
+    dw: 'DataWarehouse' = None,
+) -> pd.DataFrame:
     """
     Jinja Func to materialize a chain as a temporary view before running a query
     """
@@ -270,10 +288,11 @@ def _run_query(
             drop_sql = f"DROP VIEW IF EXISTS {source_table}"
             dw.execute_query(drop_sql, response='none', acknowledge_risk=True)
 
+
 def _set_create_statement(
-        table_type: str,
-        fqtn: str,
-    ) -> str:
+    table_type: str,
+    fqtn: str,
+) -> str:
     """
     Returns a create statement or a blank string
     """
@@ -282,10 +301,11 @@ def _set_create_statement(
         return f'CREATE OR REPLACE {table_type} {fqtn} AS \n'
     return ''
 
+
 def _set_final_select_statement(
-        transform_count: int,
-        sql: str,
-    ) -> str:
+    transform_count: int,
+    sql: str,
+) -> str:
     """
     Returns a create statement or a blank string
     """
@@ -293,11 +313,29 @@ def _set_final_select_statement(
         return _collapse_cte(sql)
     return sql
 
+
+def get_columns(
+    source_table: str,
+    running_sql: str = None,
+    dw: 'DataWarehouse' = None,
+) -> Dict[str, str]:
+    """
+    Return the column names of a given table (or sql statement)
+    """
+    return {
+        row[0]: row[1]
+        for row in dw.get_schema(
+            fqtn=dw.magic_fqtn_handler(source_table, dw.default_namespace),
+            create_sql=running_sql,
+        )
+    }
+
+
 def _source_code_functions(
-        dw: 'DataWarehouse',
-        source_table: str = None,
-        running_sql: str = None
-    ) -> Dict[str, Callable]:
+    dw: 'DataWarehouse',
+    source_table: str = None,
+    running_sql: str = None,
+) -> Dict[str, Callable]:
     """
     Get custom functions to add to the template parser
     """
@@ -306,13 +344,14 @@ def _source_code_functions(
             _run_query,
             dw=dw,
             source_table=source_table,
-            running_sql=running_sql
+            running_sql=running_sql,
         ),
         "cleanse_name": _cleanse_template_symbol,
         "raise_exception": _raise_exception,
+        "get_columns": functools.partial(get_columns, dw=dw, running_sql=running_sql),
         "itertools": {
             "combinations": combinations,
             "permutations": permutations,
-            "product": product
-        }
+            "product": product,
+        },
     }
